/***************************************************************************
 *   Copyright (C) 2008 by Georg Hennig                                    *
 *   georg.hennig@web.de                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qimage.h>
#include <qwmatrix.h>
#include <kimageio.h>
#include <libkexiv2/kexiv2.h>

using namespace KExiv2Iface;

#include "rawthumbcreator.h"

extern "C" {
	KDE_EXPORT ThumbCreator *new_creator() {
		KImageIO::registerFormats();
		return new RawThumbCreator;
  }
}

bool RawThumbCreator::create( const QString &path, int width, int height, QImage &img )
{
	if ( path.isEmpty() ) // remote file
		return false;

	KExiv2 my_kexiv2( path );

	img = my_kexiv2.getExifThumbnail( false );

	if( img.isNull() ) return false;

	img.smoothScale( width, height, QImage::ScaleMax );

	long orientation;
	if ( my_kexiv2.getExifTagLong( "Exif.Image.Orientation", orientation ) )
	{
		QWMatrix M;
		QWMatrix flip= QWMatrix(-1,0,0,1,0,0);
		switch( orientation )
		{
			case 2: M = flip; break;
			case 4: M = flip;
			case 3: M.rotate(180); break;
			case 5: M = flip;
			case 6: M.rotate(90); break;
			case 7: M = flip;
			case 8: M.rotate(270); break;
		}
		img = img.xForm(M);
	}

  if (img.depth() != 32) img = img.convertDepth(32);

  return true;
}

ThumbCreator::Flags RawThumbCreator::flags() const
{
  return None;
}
